import json
import os
import argparse
import sys

def ensure_json_extension(name):
    """
    Ensures that the filename ends with '.json'.
    
    :param name: Base name of the file.
    :return: Filename with '.json' extension.
    """
    if not name.lower().endswith('.json'):
        return name + '.json'
    return name

def sanitize_filename(name):
    """
    Sanitizes the preset name to create a valid filename by removing or replacing invalid characters,
    while preserving spaces.
    
    :param name: Preset name.
    :return: Sanitized filename.
    """
    # Define valid characters (including space)
    valid_chars = "-_.() abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    sanitized = ''.join(c if c in valid_chars else '_' for c in name)
    # Do NOT replace spaces with underscores to preserve them
    return sanitized

def export_preset(source_filename, preset_name, output_directory=None):
    """
    Exports a specific preset from the source JSON file to a new JSON file.
    
    :param source_filename: Base name of the source JSON file (without '.json').
    :param preset_name: Name of the preset to export.
    :param output_directory: Directory to save the exported preset JSON file. Defaults to current directory.
    """
    # Append '.json' extension to source filename
    source_file = ensure_json_extension(source_filename)
    
    # Verify that the source file exists
    if not os.path.isfile(source_file):
        print(f"Error: The file '{source_file}' does not exist.")
        sys.exit(1)
    
    # Read the source JSON file
    try:
        with open(source_file, 'r') as f:
            data = json.load(f)
    except json.JSONDecodeError as e:
        print(f"Error: Failed to parse JSON file '{source_file}'. {e}")
        sys.exit(1)
    except IOError as e:
        print(f"Error: Unable to read file '{source_file}'. {e}")
        sys.exit(1)
    
    # Check if 'parameterSets' key exists
    if 'parameterSets' not in data:
        print("Error: The JSON structure is invalid. Missing 'parameterSets' key.")
        sys.exit(1)
    
    # Check if the specified preset exists
    if preset_name not in data['parameterSets']:
        print(f"Error: Preset '{preset_name}' not found in the file '{source_file}'.")
        sys.exit(1)
    
    # Extract the desired preset
    preset_data = data['parameterSets'][preset_name]
    
    # Create the new JSON structure
    new_data = {
        "parameterSets": {
            preset_name: preset_data
        },
        "fileFormatVersion": data.get("fileFormatVersion", "1")  # Retain original file format version
    }
    
    # Sanitize the preset name for the filename (preserving spaces)
    safe_name = sanitize_filename(preset_name)
    new_file_name = f"{safe_name}.json"
    
    # Determine the output path
    if output_directory:
        os.makedirs(output_directory, exist_ok=True)  # Create directory if it doesn't exist
        new_file_path = os.path.join(output_directory, new_file_name)
    else:
        new_file_path = new_file_name  # Current directory
    
    # Write the new JSON to the file
    try:
        with open(new_file_path, 'w') as f:
            json.dump(new_data, f, indent=4)
        print(f"Success: Preset '{preset_name}' has been exported to '{new_file_path}'.")
    except IOError as e:
        print(f"Error: Failed to write to file '{new_file_path}'. {e}")
        sys.exit(1)

def main():
    # Set up argument parsing
    parser = argparse.ArgumentParser(description="Export a specific preset from a JSON file.")
    parser.add_argument('source_name', help="Name of the source JSON file (without '.json'). Must be enclosed in double quotes.")
    parser.add_argument('preset_name', help="Name of the preset to export. Must be enclosed in double quotes.")
    parser.add_argument('-o', '--output', help="Directory to save the exported preset JSON file. Defaults to the current directory.", default=None)
    
    args = parser.parse_args()
    
    export_preset(args.source_name, args.preset_name, args.output)

if __name__ == "__main__":
    main()
