import json
import os
import argparse
import sys

def ensure_json_extension(name):
    """
    Ensures that the filename ends with '.json'.
    
    :param name: Base name of the file.
    :return: Filename with '.json' extension.
    """
    if not name.lower().endswith('.json'):
        return name + '.json'
    return name

def load_json(file_path):
    """
    Loads a JSON file and returns its content.
    
    :param file_path: Path to the JSON file.
    :return: Parsed JSON data as a dictionary.
    """
    if not os.path.isfile(file_path):
        print(f"Error: The file '{file_path}' does not exist.")
        sys.exit(1)
    
    try:
        with open(file_path, 'r') as f:
            data = json.load(f)
        return data
    except json.JSONDecodeError as e:
        print(f"Error: Failed to parse JSON file '{file_path}'. {e}")
        sys.exit(1)
    except IOError as e:
        print(f"Error: Unable to read file '{file_path}'. {e}")
        sys.exit(1)

def save_json(data, file_path):
    """
    Saves a dictionary to a JSON file.
    
    :param data: Dictionary to save.
    :param file_path: Path to the JSON file.
    """
    try:
        with open(file_path, 'w') as f:
            json.dump(data, f, indent=4)
        print(f"Success: Merged presets have been saved to '{file_path}'.")
    except IOError as e:
        print(f"Error: Failed to write to file '{file_path}'. {e}")
        sys.exit(1)

def merge_presets(source_data, target_data, duplicate_action='prompt'):
    """
    Merges presets from source_data into target_data.
    
    :param source_data: Dictionary containing source presets.
    :param target_data: Dictionary containing target presets.
    :param duplicate_action: Action to take when a duplicate is found ('overwrite', 'skip', 'rename', 'prompt').
    :return: Updated target_data with merged presets.
    """
    source_presets = source_data.get('parameterSets', {})
    if not source_presets:
        print("Warning: No 'parameterSets' found in the source file.")
        return target_data
    
    target_presets = target_data.get('parameterSets', {})
    
    for preset_name, preset_content in source_presets.items():
        if preset_name in target_presets:
            if duplicate_action == 'overwrite':
                target_presets[preset_name] = preset_content
                print(f"Overwritten preset '{preset_name}'.")
            elif duplicate_action == 'skip':
                print(f"Skipped preset '{preset_name}'.")
                continue
            elif duplicate_action == 'rename':
                new_name = preset_name + "_new"
                while new_name in target_presets:
                    new_name += "_1"
                target_presets[new_name] = preset_content
                print(f"Added preset as '{new_name}'.")
            elif duplicate_action == 'prompt':
                print(f"\nPreset '{preset_name}' already exists in the target file.")
                action = ''
                while action.lower() not in ['o', 's', 'r']:
                    action = input("Choose an action - [O]verwrite, [S]kip, [R]ename: ").strip().lower()
                
                if action == 'o':
                    target_presets[preset_name] = preset_content
                    print(f"Overwritten preset '{preset_name}'.")
                elif action == 's':
                    print(f"Skipped preset '{preset_name}'.")
                    continue
                elif action == 'r':
                    new_name = input("Enter a new name for the preset: ").strip()
                    while new_name in target_presets:
                        print(f"The name '{new_name}' already exists in the target file.")
                        new_name = input("Please enter a different name: ").strip()
                    target_presets[new_name] = preset_content
                    print(f"Added preset as '{new_name}'.")
        else:
            target_presets[preset_name] = preset_content
            print(f"Added preset '{preset_name}'.")
    
    target_data['parameterSets'] = target_presets
    return target_data

def initialize_target(file_path):
    """
    Initializes the target JSON structure if the file does not exist.
    
    :param file_path: Path to the target JSON file.
    :return: Initialized target data dictionary.
    """
    if os.path.isfile(file_path):
        return load_json(file_path)
    else:
        print(f"Target file '{file_path}' does not exist. Creating a new one.")
        return {
            "parameterSets": {},
            "fileFormatVersion": "1"
        }

def main():
    # Set up argument parsing
    parser = argparse.ArgumentParser(description="Merge presets from a source JSON file into a target JSON file.")
    parser.add_argument('source_name', help="Name of the source preset (without '.json').")
    parser.add_argument('target_name', help="Name of the target preset (without '.json').")
    parser.add_argument('-b', '--backup', help="Create a backup of the target file before merging.", action='store_true')
    parser.add_argument('-a', '--action', choices=['overwrite', 'skip', 'rename', 'prompt'], default='prompt',
                        help="Action to take when a duplicate preset is found: overwrite, skip, rename, or prompt (default).")
    
    args = parser.parse_args()
    
    # Construct filenames by ensuring '.json' extension
    source_filename = ensure_json_extension(args.source_name)
    target_filename = ensure_json_extension(args.target_name)
    
    # Load source data
    source_data = load_json(source_filename)
    
    # Initialize or load target data
    target_data = initialize_target(target_filename)
    
    # Create a backup if requested and target file exists
    if args.backup and os.path.isfile(target_filename):
        backup_path = target_filename + ".backup"
        try:
            with open(target_filename, 'r') as original, open(backup_path, 'w') as backup:
                backup.write(original.read())
            print(f"Backup created at '{backup_path}'.")
        except IOError as e:
            print(f"Error: Failed to create backup. {e}")
            sys.exit(1)
    
    # Merge presets
    updated_target = merge_presets(source_data, target_data, args.action)
    
    # Save the merged data to the target file
    save_json(updated_target, target_filename)

if __name__ == "__main__":
    main()
